from .kafka_producer import Producer
from wedeliver_core.helpers.topics import Topics



def send_sms(mobile, message):
    response = dict(
        message=message,
        mobile=mobile
    )
    Producer().send_topic(Topics.SEND_SMS.value, response)



from functools import wraps

from flask import request

from wedeliver_core.helpers.auth import verify_user_token
from wedeliver_core.helpers.exceptions import (
    AppValidationError,
    AppMissingAuthError,
)
from wedeliver_core.helpers.acl_enum import Permission


def handle_auth(require_auth, append_auth_args=None, allowed_roles=None, allowed_permissions=None):
    def factory(func):
        @wraps(func)
        def inner_function(*args, **kws):
            if not require_auth:
                return func(*args, **kws)

            if "Authorization" not in request.headers:
                raise AppMissingAuthError("Missing authentication")

            token = request.headers["Authorization"]
            if "country_code" not in request.headers and request.endpoint != "health_check":
                raise AppValidationError("Country Code is Required (c)")

            user = verify_user_token(token=token,allowed_permissions=allowed_permissions)

            if append_auth_args and isinstance(append_auth_args, list):
                for arg in append_auth_args:
                    if not kws.get('appended_kws'):
                        kws['appended_kws'] = dict()

                    kws['appended_kws'][arg] = user.get(arg)

            return func(*args, **kws)

        return inner_function

    return factory



from wedeliver_core import WeDeliverCore

service_endpoint = dict(
    local='http://localhost:5000',
    development='http://{}-service.services',
    staging='http://{}-service.services',
    sandbox='http://{}-service-sandbox.sandbox',
    production='http://{}-service.services',
)


class ServiceConfig(object):
    name = None
    url = None

    def __init__(self, name):

        self.name = name

    def initialize(self):
        app = WeDeliverCore.get_app()

        url = service_endpoint.get(app.config.get('FLASK_ENV'))

        if url:
            self.url = url.format(self.name.replace('_SERVICE', '').lower())

        return self



from wedeliver_core import Topics, Producer


def log_model_changes(
        changes
):
    data = dict(
        topic=Topics.LOG_MODEL_CHANGES.value,
        # todo: why I should pass this. I see the finance consumer required it.
        payload=changes,
        # token=token,
    )

    Producer().send_topic(
        topic=Topics.LOG_MODEL_CHANGES.value, datajson=data
    )



from wedeliver_core import WeDeliverCore


def get_prefix(service_url):
    app = WeDeliverCore.get_app()
    final_url = ""
    service_name = app.config.get("SERVICE_NAME")
    if service_name:
        final_url += "/{}".format(service_name)

    final_url += service_url

    app.logger.debug(final_url)

    return final_url



from wedeliver_core import WeDeliverCore


class Transactions(object):
    """

    """
    session = None

    def __init__(self):
        app = WeDeliverCore.get_app()

        self.session = app.extensions['sqlalchemy'].db.session

    @staticmethod
    def atomic():
        """

        :return:
        """
        return Transactions()

    def commit(self, instance):
        """

        :param instance:
        """
        self.session.add(instance)
        self.session.flush()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if exc_type:
            self.session.rollback()
        else:
            self.session.commit()
        self.session.close()

# with Transaction.atomic() as session:
#     # Insert into table 1
#     session.add(Table1(...))
#     session.flush()
#
#     # Insert into table 2
#     session.add(Table2(...))
#     session.flush()



class WeDeliverCore:
    __app = None

    @staticmethod
    def get_app():
        """ Static access method. """
        if WeDeliverCore.__app == None:
            WeDeliverCore()
        return WeDeliverCore.__app

    def __init__(self, app=None):
        """ Virtually private constructor. """
        if WeDeliverCore.__app != None:
            raise Exception("This class is a singleton!")
        else:
            WeDeliverCore.__app = app
            setup_default_routes(app)


def setup_default_routes(app):
    from wedeliver_core.app_decorators.app_entry import route
    from wedeliver_core.helpers.fetch_relational_data import fetch_relational_data
    @route(
        path='/',
        require_auth=False
    )
    def _health_check_service():
        return dict(name="{} Service".format(app.config.get('SERVICE_NAME')), works=True)

    @route(
        path='/health_check',
        require_auth=False
    )
    def _health_check_with_path_service():
        return dict(name="{} Service".format(app.config.get('SERVICE_NAME')), works=True)

    @route("/fetch_relational_data", methods=["POST"], require_auth=False)
    def _fetch_relational_data_service(validated_data):
        """
        Swagger definition
        """
        return fetch_relational_data(**validated_data)
